#-------------------------------------------------------------------------------------------------#
# file  : remove_shs_duplicates.py
# date  : November 28, 2014
# author: Julien OSMALSKYJ
# email : josmalsky@ulg.ac.be
#
# This module defines a parse_file function which loads either the SHS dataset or the
# duplicate list coming with the Million Song Dataset (MSD). The format of the input
# file is the following :
#
# % Clique 1 title
#   TRACK_ID_001
#   TRACK_ID_002
# % Clique 2 title
#   TRACK_ID_042
#   TRACK_ID_052
#   TRACK_ID_062
#
# The function returns a list of cliques of the following form :
# [[Title1, ID1, ID2], [Title2, ID3, ID4, ID4], ...]
#
# and a dictionary in which the key is a Track ID, and the value is the number of the clique
# in which that track is stored.
#-------------------------------------------------------------------------------------------------#
def parse_file(path):
    dataset = open(path)
    cliques = []    
    cover = []
    hash_dic = {}

    for line in dataset: 
        if line[0] == "%":
            if len(cover) != 0:
                cliques.append(cover)
                            
            title = line[line.rfind('%') + 1:].lstrip(' ').rstrip('\n')            
            cover = [title]
        
        elif line[0] == "T":
            track_id = line[:line.find("<SEP>")]
            hash_dic[track_id] = len(cliques)            
            cover.append(track_id)

    dataset.close()

    if len(cover) != 0:
        cliques.append(cover)

    return cliques, hash_dic