#-------------------------------------------------------------------------------------------------#
# file  : remove_shs_duplicates.py
# date  : November 28, 2014
# author: Julien OSMALSKYJ
# email : josmalsky@ulg.ac.be
#
# This script removes duplicates from the SecondHandSong Dataset (SHSD) which is
# a subset of the Million Song Dataset (MSD). 
# An official list of duplicates is provided at
# http://labrosa.ee.columbia.edu/millionsong/blog/11-3-15-921810-song-dataset-duplicates.
# The SHSD can be found at the following address:
# http://labrosa.ee.columbia.edu/millionsong/secondhand
# 
# If a clique of the SHSD contains two duplicate songs, it may lead to optimistic results
# in some MIR tasks, e.g. cover songs recognition.
#
# This code is released under GNU Public License.
#-------------------------------------------------------------------------------------------------#

import sys
import cPickle as pkl
import cliques_parser as parser

#-------------------------------------------------------------------------------------------------#
# Return TRUE if the input track is found in one SHS clique.
#-------------------------------------------------------------------------------------------------#
def found(track, dic):
	return dic.get(track) != None

#-------------------------------------------------------------------------------------------------#
# Remove a track from the SHS dataset
#-------------------------------------------------------------------------------------------------#			
def remove(track, shs, dic):
	clique_num = dic.get(track)

	if clique_num != None:
		for t in shs[clique_num]:
			if t == track:
				print "Removing " + t + " from clique " + str(clique_num)
				shs[clique_num].remove(t)	

#-------------------------------------------------------------------------------------------------#
# Prune the SHS dataset by removing duplicates
#-------------------------------------------------------------------------------------------------#	
def prune(shs_dataset, dup_dataset, shs_dic):
	for dup_clique in dup_dataset:
		tracks = dup_clique[1:] # first item is the name of the clique

		for track in tracks:
			temp = track

			if found(track, shs_dic):
				# in that case, keep the found track and remove all the other tracks
				# of the duplicate clique from the SHS to keep only one version.
				for t in tracks:
					if t != temp:									
						remove(t, shs_dataset, shs_dic)

#-------------------------------------------------------------------------------------------------#
# Main code 
#
# Usage: python ./remove_shs_duplicates [SHS_FULL_INPUT] [DUPLICATES FILE] [OUTPUT FILE]
#
#-------------------------------------------------------------------------------------------------#
shs_path = sys.argv[1] # shs input file
dup_path = sys.argv[2] # dup input file
out_path = sys.argv[3] # path for output cleaned shs

shs_dataset, shs_dic = parser.parse_file(shs_path)
dup_dataset, dup_dic = parser.parse_file(dup_path)
shs_size = len(shs_dataset)
dup_size = len(dup_dataset)

print "SHS loaded: " + str(shs_size) + " cliques."
print "DUP loaded: " + str(dup_size) + " cliques."

# cleaning algorithm
print "Processing data."
prune(shs_dataset, dup_dataset, shs_dic)

# remove cliques containing less than 2 tracks
for clique in shs_dataset:
	if len(clique[1:]) < 2:
		shs_dataset.remove(clique)

print "Old SHS size: " + str(shs_size)
print "New SHS size: " + str(len(shs_dataset))

# write updated SHS in output file
out_file = open(out_path, 'w')

for clique in shs_dataset:
	title = clique[0]
	tracks = clique[1:]

	out_file.write("%" + title + "\n")

	for track in tracks:
		out_file.write(track + "\n")

print "New dataset written in " + out_path
out_file.close()
